@@ -620,9 +620,14 @@ def get_color_from_float(in_attr_dict):
 ## get the names of all attributes
 def get_attrs():
     global large_comp_G
-    for i in iter(large_comp_G.node.items()):
-        first_node = i[1]
-        break
+    if "node" in dir(large_comp_G):
+        for i in iter(large_comp_G.node.items()):
+            first_node = i[1]
+            break
+    else:
+        for i in iter(large_comp_G.nodes.items()):
+            first_node = i[1]
+            break
     return(list(first_node.keys()))
 #####################
 
