@@ -167,6 +167,7 @@ def pyminer_analysis(infile=None,
                     leave_mito_ribo = False,
                     do_dispersion = True,
                     sc_clust = False,
+                    num_pos_cor_cutoff = 10,
                     pre_determined_sample_k=0,
                     sample_cluster_iter = 10,
                     var_norm_for_clust = True,
@@ -184,6 +185,7 @@ def pyminer_analysis(infile=None,
                     processes = None,
                     rho_cutoff = None,
                     prop = False, 
+                    log_odds = False,
                     block_size = 5000,
                     FPR_multiple = 15,
                     noi = None,
@@ -261,7 +263,7 @@ def pyminer_analysis(infile=None,
             sys.exit('please use a rho cutoff between zero and one. We recommend not using this argument at all to allow for the built in False Positive Rate (FPR) algorithm to take over. If using scRNAseq, try something in the range of 0.25-0.35. For bulk RNAseq, try something closer to 0.70 - 0.90.')
 
     out_adj = os.path.join(temp,'adj_list_rho.tsv')
-    adj_call_string = 'mat_to_adj_list.py -i '+infile+' -o '+out_adj+' -id_list '+os.path.join(temp,'ID_list.txt ')
+    adj_call_string = 'mat_to_adj_list.py -i '+infile+' -o '+out_adj+' -id_list '+os.path.join(temp,'ID_list.txt ')+' -num_pos_cor_cutoff '+str(num_pos_cor_cutoff)
     if rho_cutoff!=None:
         adj_call_string+=' -rho '+str(rho_cutoff)
     if do_h5_py:
@@ -271,6 +273,9 @@ def pyminer_analysis(infile=None,
         adj_call_string += ' -sc_clust '
     if prop:
         adj_call_string += ' -prop '
+    if log_odds:
+        adj_call_string += ' -log_odds '
+    print("LOG ODDS:",log_odds)
     adj_call_string += " -FPR_multiple "+str(FPR_multiple)
     adj_call_string += ' -rand_seed '+str(rand_seed)
     all_calls.append(adj_call_string)
@@ -282,50 +287,8 @@ def pyminer_analysis(infile=None,
     pos_neg_count = np.zeros((len(IDlist),2))
 
     out_adj_pos = out_adj[:-4]+'_dedup_pos.tsv'
-    rm(out_adj_pos)
-    pos_f_out=open(out_adj_pos,'a')
-
     out_adj_neg = out_adj[:-4]+'_dedup_neg.tsv'
-    rm(out_adj_neg)
-    neg_f_out=open(out_adj_neg,'a')
 
-    counter = 0
-    first = True
-    for line in fileinput.input(out_adj[:-4]+'_dedup.tsv'):
-        if first:
-            print('reading ',out_adj[:-4]+'_dedup.tsv')
-            #print(float(temp_line[-1]))
-            pos_f_out.write(line)
-            neg_f_out.write(line)
-            first = False
-        else:
-            temp_line = strip_split(line)
-            #print(float(temp_line[-1]))
-            try:
-                float(temp_line[-1])
-            except:
-                print('error at line:',counter)
-                print('\t',line)
-            if float(temp_line[-1])>0:
-                pos_f_out.write(line)
-                pos_neg_count[ID_hash[temp_line[0]]][0]+=1
-                pos_neg_count[ID_hash[temp_line[1]]][0]+=1
-            else:
-                neg_f_out.write(line)
-                pos_neg_count[ID_hash[temp_line[0]]][1]+=1
-                pos_neg_count[ID_hash[temp_line[1]]][1]+=1
-        counter+=1
-    pos_f_out.close()
-    neg_f_out.close()
-
-    ## log some stats about how many positive and negative relationships there are
-    pos_neg_count = pos_neg_count.tolist()
-    for i in range(0,len(pos_neg_count)):
-        pos_neg_count[i] = [IDlist[i]] + pos_neg_count[i]
-
-    pos_neg_count = [['ID','num_pos_cor','num_neg_cor']]+ pos_neg_count
-
-    write_table(pos_neg_count, os.path.join(temp,'positive_negative_cor_counts.txt'))
 
     ################################################
     ## do feature selection
