@@ -14,6 +14,7 @@ try:
     from scipy._lib._version import NumpyVersion
 except:
     pass
+import psutil
 from numpy import array, asarray, ma, zeros, var, average
 import multiprocessing
 from multiprocessing.dummy import Pool as ThreadPool
@@ -25,33 +26,12 @@ from copy import deepcopy
 import h5py
 from math import floor
 from scipy.stats import mstats_basic, rankdata
+from scipy.sparse import csr_matrix
+import ray
 
-#from . import distributions
-#from . import mstats_basic
-#from ._distn_infrastructure import _lazywhere
-#from ._stats_mstats_common import _find_repeats, linregress, theilslopes
-#from ._stats import _kendall_condis
 
 __all__ = []
 
-# __all__ = ['find_repeats', 'gmean', 'hmean', 'mode', 'tmean', 'tvar',
-#            'tmin', 'tmax', 'tstd', 'tsem', 'moment', 'variation',
-#            'skew', 'kurtosis', 'describe', 'skewtest', 'kurtosistest',
-#            'normaltest', 'jarque_bera', 'itemfreq',
-#            'scoreatpercentile', 'percentileofscore', 'histogram',
-#            'histogram2', 'cumfreq', 'relfreq', 'obrientransform',
-#            'signaltonoise', 'sem', 'zmap', 'zscore', 'iqr', 'threshold',
-#            'sigmaclip', 'trimboth', 'trim1', 'trim_mean', 'f_oneway',
-#            'pearsonr', 'fisher_exact', 'spearmanr', 'pointbiserialr',
-#            'kendalltau', 'linregress', 'theilslopes', 'ttest_1samp',
-#            'ttest_ind', 'ttest_ind_from_stats', 'ttest_rel', 'kstest',
-#            'chisquare', 'power_divergence', 'ks_2samp', 'mannwhitneyu',
-#            'tiecorrect', 'ranksums', 'kruskal', 'friedmanchisquare',
-#            'chisqprob', 'betai',
-#            'f_value_wilks_lambda', 'f_value', 'f_value_multivariate',
-#            'ss', 'square_of_sums', 'fastsort', 'rankdata',
-#            'combine_pvalues', ]
-
 
 def _chk_asarray(a, axis):
     if axis is None:
@@ -114,7 +94,7 @@ def dense_rank(a):
     return(rankdata(a,method="dense"))
     
 
-def no_p_spear(a, b=None, axis=0, nan_policy='propagate', rm_nans_by_col = False, prop = False):
+def no_p_spear(a, b=None, axis=0, nan_policy='propagate', rm_nans_by_col = False, prop = False, log_odds = False, shuffle = False, log_odds_correction = 0):
     a=make_dense(a)
     b=make_dense(b)
 
@@ -132,9 +112,10 @@ def no_p_spear(a, b=None, axis=0, nan_policy='propagate', rm_nans_by_col = False
         return no_p_spear(a, b, axis)
     
     if a.size <= 1:
-        return SpearmanrResult(np.nan, np.nan)
+        return(np.nan, np.nan)
     #ar = np.apply_along_axis(rankdata, axisout, a)
-    ar = np.apply_along_axis(dense_rank, axisout, a)
+    if not log_odds:
+        ar = np.apply_along_axis(dense_rank, axisout, a)
     
     br = None
     if b is not None:
@@ -147,12 +128,19 @@ def no_p_spear(a, b=None, axis=0, nan_policy='propagate', rm_nans_by_col = False
             return no_p_spear(a, b, axis)
         
         #br = np.apply_along_axis(rankdata, axisout, b)
-        br = np.apply_along_axis(dense_rank, axisout, b)
+        if not log_odds:
+            br = np.apply_along_axis(dense_rank, axisout, b)
     n = a.shape[axisout]
-    if not prop:
-        rs = np.corrcoef(ar, br, rowvar=axisout)
-    else:
+    if prop:
         rs = pp_equivalent_to_cov(ar, br)
+    elif log_odds:
+        rs = log_odds_equivalent_to_cov(a, b, shuffle = shuffle, log_odds_correction = log_odds_correction)
+        print("max of no_p_spear:",np.max(rs))
+        return(rs)#, log_odds_correction = log_odds_correction)
+    else:
+        ## finally, the default is spearman
+        print("\tgetting spearman correlation")
+        rs = np.corrcoef(ar, br, rowvar=axisout)
     
     olderr = np.seterr(divide='ignore')  # rs can have elements equal to 1
     try:
@@ -210,6 +198,268 @@ def fast_single_spear(a,b,shuffle = False):
 #     else:
 #         return rs
 
+########################################################
+########################################################
+########################################################
+########## bayesean co-occurance functions #############
+########################################################
+########################################################
+########################################################
+# def get_log_odds_mat(a, b):
+#     ## should try to make a faster version...
+#     v1_and_v2 = a * b[:,:,None]
+#     ####....
+#     v1_only = np.sum((vect1 ^ vect2)==1)## ^ is xor instead of - for boolean
+#     v2_only = np.sum((vect2 ^ vect1)==1)## ^ is xor instead of - for boolean
+#     neither = np.sum((vect2 + vect1)==0)
+#     #proportion_mat = np.array([[v1_only,v1_and_v2],[neither,v2_only]])
+#     #log_odds_ratio = np.log((v1_and_v2*neither)/(v1_only*v2_only))
+#     log_odds_ratio = np.log10(((v1_and_v2+1)*(neither+1))/((v1_only+1)*(v2_only+1)))
+def get_linear_triangle(in_mat, bin_size):
+    upper_tiangle_indices = np.triu_indices(bin_size,k=1)
+    ## this means that we just have a single negative rho cutoff for all genes
+    ## subset the negative ones
+    linear_rho = np.array(in_mat[upper_tiangle_indices]).squeeze()
+    return(linear_rho)
+
+
+
+def get_log_odds(vect1, vect2, already_bool = False, i=0 , j=0, shuffle = False):
+    ## Returns the UNCORRECTED log odds
+    if not already_bool:
+        if type(vect1)!=np.ndarray:
+            vect1 = np.array(vect1)
+        if type(vect2)!=np.ndarray:
+            vect2 = np.array(vect2)
+        vect1 = vect1.squeeze()
+        vect2 = vect2.squeeze()
+        vect1_bool = np.array(vect1>0,dtype = bool)
+        vect2_bool = np.array(vect2>0,dtype = bool)
+    else:
+        vect1_bool = vect1.squeeze()
+        vect2_bool = vect2.squeeze()
+    if shuffle:
+        np.random.shuffle(vect1_bool)
+        np.random.shuffle(vect2_bool)
+    v1_and_v2 = np.sum(vect1_bool * vect2_bool)
+    v1_only = np.sum(np.array(vect1_bool==1, dtype = bool) * np.array(vect2_bool==0, dtype = bool))
+    v2_only = np.sum(np.array(vect2_bool==1, dtype = bool) * np.array(vect1_bool==0, dtype = bool))
+    # if (v1_and_v2+v1_only==1) or (v1_and_v2+v2_only==1):
+    #     return(0)
+    neither = np.sum((vect1_bool + vect1_bool)==0)
+    #proportion_mat = np.array([[v1_only,v1_and_v2],[neither,v2_only]])
+    #log_odds_ratio = np.log((v1_and_v2*neither)/(v1_only*v2_only))
+    log_odds_ratio = np.log2(((v1_and_v2+1)*(neither+1))/((v1_only+1)*(v2_only+1)))
+    # if v1_and_v2 == 0 and log_odds_ratio>0:
+    #     return(0)
+    ###################
+    ## calculate expected
+    total_events = v1_and_v2+v1_only+v2_only+neither
+    v1_percent = (v1_and_v2+v1_only)/total_events
+    v2_percent = (v1_and_v2+v2_only)/total_events
+    v1_and_v2_expected = total_events * (v1_percent * v2_percent)
+    v1_only_expected = v1_percent*total_events - v1_and_v2_expected
+    v2_only_expected = v2_percent*total_events - v1_and_v2_expected
+    neither_expected = total_events - (v1_and_v2_expected + v1_only_expected + v2_only_expected)
+    log_odds_ratio_expected = np.log10(((v1_and_v2_expected+1)*(neither_expected+1))/((v1_only_expected+1)*(v2_only_expected+1)))
+    if False:#log_odds_ratio > 10:
+        pass
+    ###
+    delta_log_odds = log_odds_ratio - log_odds_ratio_expected
+    if np.isnan(delta_log_odds):
+        print("i", i)
+        print("j", j)
+        print("v1_and_v2",v1_and_v2)
+        print("v1_only", v1_only)
+        print("v2_only", v2_only)
+        print("neither", neither)
+        print("vect1_bool", np.sum(vect1_bool), vect1_bool)
+        print("vect2_bool", np.sum(vect2_bool), vect2_bool)
+        print("log_odds_ratio", log_odds_ratio)
+        print("v1_and_v2_expected",v1_and_v2_expected)
+        print("v1_only", v1_only_expected)
+        print("v2_only", v2_only_expected)
+        print("neither", neither_expected)
+        print("log_odds_ratio_expected", log_odds_ratio_expected)
+        return(0)
+    else:
+        return(delta_log_odds)
+    # ############################
+    # ## thought about implementing something like a lod score, but haven't tested it yet
+    # #R = v1_and_v2+neither +1
+    # #NR = v1_only+v2_only +1
+    # #print(R, NR)
+    # #lod_score = np.log10(((R/(R+NR))**R)*((1-(R/(R+NR)))**NR) / 0.5**(NR+R))
+    # return(delta_log_odds)#log_odds_ratio)#, lod_score)
+
+
+@ray.remote
+def ray_log_odds(c, index_pairs, start, end, shuffle = False):
+    #print(start, end)
+    num_compare = end-start
+    job_results = np.zeros(num_compare)
+    #job_results = []
+    for n in range(start, end):
+        i = index_pairs[0][n]
+        j = index_pairs[1][n]
+        #job_results.append("n:"+str(n)+","+str(i)+"_"+str(j))
+        #job_results.append(get_log_odds(c[i],c[j]))
+        job_results[n-start]=get_log_odds(c[i],c[j], already_bool = True, i=i, j=j, shuffle = shuffle)
+        # if job_results[n-start] > 15:
+        #     print("\t",job_results[n-start], i,j)
+        #print(num_compare, n)
+        #print(job_results[n])
+    #print("max ray_log_odds:",np.max(job_results))
+    return(job_results)
+
+
+def do_ray_log_odds(a, b=None, threads = None, verbose = False, shuffle = False):
+    if b is None:
+        c = a
+    else:
+        ## concatenate them
+        c = np.concatenate((a,b))
+    c = np.array(c>0,dtype = bool)
+    ## then figure out the pairs to do
+    index_pairs = np.triu_indices(c.shape[0], k=1)## k=1 for offsetting so we don't self-compare
+    ## allocate the pairs to jobs
+    if threads == None:
+        threads = min([index_pairs[0].shape[0], multiprocessing.cpu_count()-1])
+    events_per_bin = int(index_pairs[0].shape[0]/threads)
+    end_indices = [(i+1)*events_per_bin for i in range(int(threads))] + [index_pairs[0].shape[0]]
+    start_indices = [0] + end_indices[:-1]
+    if verbose:
+        print("start_indices:", start_indices)
+        print("end_indices:", end_indices)
+        print("index pairs:", index_pairs)
+        print("num pairs:", index_pairs[0].shape[0])
+    ## start ray and allocate the jobs
+    mem = psutil.virtual_memory()
+    used_mem = mem[2]## 2 is used
+    #psutil.virtual_memory()[2]## 2 is used
+    process_mem = 6#Gigs
+    total_mem = 24
+    print("getting delta_log_odds")
+    ray.init(memory = int(total_mem*1000000000), object_store_memory=int(process_mem*1000000000))
+    ray_c = ray.put(c)
+    ray_jobs = []
+    for thread in range(threads+1):
+        ray_jobs.append(ray_log_odds.remote(ray_c, index_pairs, start_indices[thread], end_indices[thread], shuffle = shuffle))
+    ray_sults = ray.get(ray_jobs)
+    ray.shutdown()
+    data = np.concatenate(ray_sults)
+    #return(data, index_pairs[0], index_pairs[1])
+    final_results = csr_matrix((data, (index_pairs[0],index_pairs[1])), shape=(c.shape[0], c.shape[0])).toarray()
+    final_results += final_results.T
+    print("max of do_ray_log_odds:",np.max(final_results))
+    return(final_results)
+
+
+
+
+
+def log_odds_equivalent_to_cov(m, 
+                               y=None, 
+                               rowvar=True, 
+                               bias=False, 
+                               ddof=None, 
+                               fweights=None,
+                               aweights=None,
+                               log_odds_correction = 0,
+                               shuffle = False):
+    # Check inputs
+    if ddof is not None and ddof != int(ddof):
+        raise ValueError("ddof must be integer")
+    # Handles complex arrays too
+    m = np.asarray(m)
+    if m.ndim > 2:
+        raise ValueError("m has more than 2 dimensions")
+    if y is None:
+        dtype = np.result_type(m, np.float64)
+    else:
+        y = np.asarray(y)
+        if y.ndim > 2:
+            raise ValueError("y has more than 2 dimensions")
+        dtype = np.result_type(m, y, np.float64)
+    X = array(m, ndmin=2, dtype=dtype)
+    if not rowvar and X.shape[0] != 1:
+        X = X.T
+    if X.shape[0] == 0:
+        return np.array([]).reshape(0, 0)
+    if y is not None:
+        y = array(y, copy=False, ndmin=2, dtype=dtype)
+        if not rowvar and y.shape[0] != 1:
+            y = y.T
+        X = np.concatenate((X, y), axis=0)
+    if ddof is None:
+        if bias == 0:
+            ddof = 1
+        else:
+            ddof = 0
+    # Get the product of frequencies and weights
+    w = None
+    if fweights is not None:
+        fweights = np.asarray(fweights, dtype=float)
+        if not np.all(fweights == np.around(fweights)):
+            raise TypeError(
+                "fweights must be integer")
+        if fweights.ndim > 1:
+            raise RuntimeError(
+                "cannot handle multidimensional fweights")
+        if fweights.shape[0] != X.shape[1]:
+            raise RuntimeError(
+                "incompatible numbers of samples and fweights")
+        if any(fweights < 0):
+            raise ValueError(
+                "fweights cannot be negative")
+        w = fweights
+    if aweights is not None:
+        aweights = np.asarray(aweights, dtype=float)
+        if aweights.ndim > 1:
+            raise RuntimeError(
+                "cannot handle multidimensional aweights")
+        if aweights.shape[0] != X.shape[1]:
+            raise RuntimeError(
+                "incompatible numbers of samples and aweights")
+        if any(aweights < 0):
+            raise ValueError(
+                "aweights cannot be negative")
+        if w is None:
+            w = aweights
+        else:
+            w *= aweights
+    avg, w_sum = average(X, axis=1, weights=w, returned=True)
+    w_sum = w_sum[0]
+    # Determine the normalization
+    if w is None:
+        fact = X.shape[1] - ddof
+    elif ddof == 0:
+        fact = w_sum
+    elif aweights is None:
+        fact = w_sum - ddof
+    else:
+        fact = w_sum - ddof*sum(w*aweights)/w_sum
+    if fact <= 0:
+        warnings.warn("Degrees of freedom <= 0 for slice",
+                      RuntimeWarning, stacklevel=3)
+        fact = 0.0
+    X -= avg[:, None]
+    if w is None:
+        X_T = X.T
+    else:
+        X_T = (X*w).T
+    c = do_ray_log_odds(X,)
+    print("max of log_odds_equivalent_to_cov1:",np.max(c))
+    return c.squeeze()-log_odds_correction
+
+
+
+
+# def bayes_co_occur_mat(vect1, vect2):
+#     vect1_bool = np.array(vect1>0,dtype = bool)
+#     vect2_bool = np.array(vect2>0,dtype = bool)
+#     v1_percent = np.sum(vect1_bool)
+
 
 ########################################################
 ########################################################
@@ -485,7 +735,7 @@ def pp_equivalent_to_cov(m, y=None, rowvar=True, bias=False, ddof=None, fweights
 ########################################################
 
 
-def get_Z_cutoff(vect, z=4.5,positive = False):
+def get_Z_cutoff(vect, z=4.5,positive = False, cap = 0.9):
     ## this is for a half distribution, so mult by -1 to make it 'normal'
     vect_original = np.array(deepcopy(vect))
     keep = np.logical_not(np.isnan(vect_original))
@@ -500,9 +750,11 @@ def get_Z_cutoff(vect, z=4.5,positive = False):
     cutoff = v_std * z
     if not positive:
         cutoff *= -1
-        cutoff = max([-0.9,cutoff])
-    else:
-        cutoff = min([0.9,cutoff])
+    if cap is not None:
+        if not positive:
+            cutoff = max([-1*cap,cutoff])
+        else:
+            cutoff = min([cap,cutoff])
     print('cutoff:',cutoff)
     if not positive:
         num_sig = np.sum(np.array(vect_original<cutoff,dtype = bool))
@@ -544,3 +796,8 @@ def get_empiric_FPR(vect, FPR=1000, positive = False):
     else:
         return(cutoff * -1)
 
+
+
+
+# def get_empiric_log_odds_FPR(vect, ):
+#     cutoff
\ No newline at end of file
